#!/usr/bin/env python3
import sys
from multiprocessing import Process
from multiprocessing.connection import Listener

from . import Database
from .uzelmumu import run


def process_query(path, cnx):
    queries = cnx.recv()
    database = Database(path)
    try:
        objects = run(database, *queries)
    except:
        print('error:', queries)
    cnx.send(objects)
    cnx.close()
    database.close()


if __name__ == '__main__':
    address = ('localhost', 6002)     # family is deduced to be 'AF_INET'
    path = sys.argv[1]

    database = Database(path)
    with database.transaction() as txn:
        amirouche = txn.vertex('user')
        amirouche['name'] = 'amirouche'

    # create feed
    with database.transaction() as txn:
        amirouche = txn.get(amirouche.identifier())

        hypermove = txn.vertex('feed')
        hypermove['url'] = 'http://hyperdev.fr/atom.xml'
        txn.edge(amirouche, 'feeds', hypermove)

        hypermove = txn.vertex('feed')
        url = 'http://www.hyperdev.fr/keyword/fran%C3%A7ais/atom.xml'
        hypermove['url'] = url
        txn.edge(amirouche, 'feeds', hypermove)
    database.close()

    print('id:', amirouche.identifier())

    print('running at {}'.format(address))
    with Listener(address) as listener:
        try:
            while True:
                with listener.accept() as cnx:
                    print('received cnx')
                    p = Process(target=process_query, args=(path, cnx))
                    p.start()
        except KeyboardInterrupt:
            listener.close()
            print('exiting')
