"""uzelmumu query engine"""
from types import MethodType
from types import GeneratorType

from .chained_methods import ChainedMethods


def graph():
    return ChainedMethods('graph')


def run(db, *queries):
    """Run query which is a serialized chained method"""
    with db.transaction() as graph:
        objects = [graph]
        # remove graph which MUST be the first argument
        for query in queries:
            query = query[1:]
            objects = run_query(query, *objects)
    return objects


def run_query(query, *objects):
    for name, args, kwargs in query:
        new_objects = list()
        for obj in objects:
            attribute = getattr(obj, name)
            if isinstance(attribute, MethodType):
                new_obj = attribute(*args, **kwargs)
            else:
                new_obj = attribute
            if isinstance(new_obj, GeneratorType):
                add = getattr(new_objects, 'extend')
            else:
                add = getattr(new_objects, 'append')
            add(new_obj)
        objects = new_objects
    return objects
